#include "PropaneActor.h"
#include <cmath>

#define PI 3.141592654

void PropaneActor::draw(int px, int py) {
	/// Draws the actor to screen using its current sprite, rotation, height, width and location, based on specified camera coordinates
	if(!myMgr) {
		fprintf(stderr, "Can't draw me without a valid SpriteManager\n");
		return;
	}

	// Find the screen location.
	int scrx = (myMgr->getScreenWidth() / 2) - (int)(px - x);
	int scry = (myMgr->getScreenHeight() / 2) - (int)(py - y);

	// Now render.
	if(scrx <= myMgr->getScreenWidth() && scrx >= -(getWidth())) {
		if(scry <= myMgr->getScreenHeight() && scry >= -(getHeight())) {
			myMgr->drawSprite(getFrame(), scrx, scry, (int)getWidth(), (int)getHeight(), (float)getRotation());
		}
	}
}

PropaneActor::~PropaneActor() { }

void PropaneActor::setXYWH(float nx, float ny, float nw, float nh) {
	x = nx;
	y = ny;
	w = nw;
	h = nh;
}

float PropaneActor::DegToRad(double degrees) {
	/// Converts degrees out of 360 to radians out of 2 PI
	return (float)(rotation * (PI / 180.0f));
}

float PropaneActor::getForwardX(float distance) {
	return x + sinf(DegToRad(rotation+90))*distance;
}

float PropaneActor::getForwardY(float distance) {
	 return y + cosf(DegToRad(rotation+90))*distance;
}

void PropaneActor::rotateTowards(double maxAngle, double goal) {
	/// rotate the target actor towards a "goal" angle, useful for gradually turning towards things
	goal = goal - rotationOutOf360();

	if(goal < 3 && goal > -3) {
		// Close enough to our target
		return;
	}

	if(goal > 180) {
		goal = 180 - goal;
	}
	if(goal < -180) {
		goal = goal + 360;
	}
	if(goal > 0) {
		// Turning left is shorter
		rotation += maxAngle;
	}
	else {
		// Turning right is shorter
		rotation -= maxAngle;
	}
}

double PropaneActor::angleTowards(float tx, float ty) {
	/// Rotate the target actor to face a specified x,y coordinate in world space.
	float dx = (x - tx);
	float dy = (y - ty);

	float theta = (float)atan2(-dy, dx);
	if(theta < 2) {
		theta = (float)(2 * PI) + theta;
	}
	return (theta * (180/PI));
}

double PropaneActor::angleDistance(double angle1, double angle2) {
	angle2 = angleOutOf360(angle2);
	angle2 -= angleOutOf360(angle1);

	if(angle2 > 180) {
		angle2 = 180 - angle2;
	}
	if(angle2 < -180) {
		angle2 = angle2 + 360;
	}
	return angle2;
}

double PropaneActor::angleOutOf360(double angle) {
	while(angle >= 360.0f) {
		angle -= 360.0f;
	}
	while(angle < 0.0f) {
		angle += 360.0f;
	}
	return angle;
}

void PropaneActor::rotateFacing(float tx, float ty) {
	rotation = angleTowards(tx,ty);
}

double PropaneActor::rotationOutOf360() {
	/// Returns the actor's rotation (in degrees) out of 360, rather than -INF to +INF
	double target = getRotation();

	while(target >= 360) {
		target -= 360;
	}
	while(target < 0) {
		target += 360;
	}

	return target;
}

bool PropaneActor::collision(PropaneActor* otherActor) {
	/// Detects a collision with another PropaneActor object. Returns true if collided.
	float left1, left2;
	float right1, right2;
	float top1, top2;
	float bottom1, bottom2;

	left1 = x;
	left2 = otherActor->getX();

	right1 = x + w;
	right2 = otherActor->getX() + otherActor->getWidth();

	top1 = y;
	top2 = otherActor->getY();

	bottom1 = y + w;
	bottom2 = otherActor->getY() + otherActor->getHeight();

	if(bottom1 < top2)
		return false;

	if(top1 > bottom2)
		return false;

	if(right1 < left2)
		return false;

	if(left1 > right2)
		return false;

	return true;
}

PropaneActor::PropaneActor(SpriteManager* mgr) {
	/// Constructs a new PropaneActor that links to the specified SpriteManager
	myMgr = mgr;

	x = 0;
	y = 0;
	w = 0;
	h = 0;

	numFrames = 0;
	rotation = 0;
	curFrame = 0;
	startFrame = 0;
	frameDelay = 0.0f;
	frameTimer = 0.0f;

	tempFrame = 0;
	inTemp = false;
}

void PropaneActor::setX(float nx) {
	/// Sets this actor's world X coordinate
	x = nx;
}

void PropaneActor::setY(float ny) {
	/// Sets this actor's world Y coordinate
	y = ny;
}

float PropaneActor::getWidth() {
	/// Returns the width of the actor sprite
	return w;
}

float PropaneActor::getHeight() {
	/// Returns the height of the actor sprite
	return h;
}

void PropaneActor::temporaryFrame(int frame) {
	/// Interrupts animation temporarily and substitutes a frame
	tempFrame = frame;
	inTemp = true;
	frameTimer = 0.0f;
}

float PropaneActor::getX() {
	/// Returns the x-location of the actor in world space.
	return x;
}

float PropaneActor::getY() {
	/// Returns the y-location of the actor in world-space.
	return y;
}

void PropaneActor::setWidth(float nw) {
	/// Set the render and collision width of the actor in world units.
	if(nw >= 0) {
		w = nw;
	}
}

void PropaneActor::setHeight(float nh) {
	/// Set the render and collision height of the actor in world units
	if(nh >= 0) {
		h = nh;
	}
}

void PropaneActor::move(float dx, float dy) {
	/// Move the actor the specified amount.
	x += dx;
	y += dy;
}

void PropaneActor::setRotation(double rot) {
	/// Sets the rotation to a given angle, in degrees
	rotation = rot;
}

double PropaneActor::getRotation() {
	/// Returns the actor's heading, in degrees
	return rotation;
}

void PropaneActor::resetAnimation() {
	/// Re-initializes the animation to default values.
	inTemp = false;
	tempFrame = 0;
	frameTimer = 0.0f;
	curFrame = 0;
}

void PropaneActor::setAnimation(int nStart, int nFrames, float nDelay) {
	/// Resets the animation's default values with provided start sprite, number of frames, and delay between frame iteration
	if(nDelay >= 0.0f) {
		frameDelay = nDelay;
	}

	if(nFrames >= 0) {
		numFrames = nFrames;
	}

	if(nStart >= 0 && nStart < myMgr->size()) {
		startFrame = nStart;
	}
}

void PropaneActor::rotate(double dRot) {
	/// Changes the rotation of the actor by the provided amount of degrees
	rotation += dRot;
}

void PropaneActor::updateAnimation(float delta) {
	/// Increments animation timer and handles frame advancement from given delta interval
	frameTimer += delta;
	if(frameTimer >= frameDelay) {
		if(inTemp) {
			// Going "out" of temporary frame.
			frameTimer = 0.0f;
			inTemp = false;
			tempFrame = 0;
			return;
		}
		// Advance animation frame.
		frameTimer = 0.0f;
		curFrame++;
		if(curFrame >= numFrames) {
			// Reset animation
			curFrame = 0;
		}
	}
}

int PropaneActor::getFrame() {
	/// Returns the current sprite that should be used to render this actor
	if(!inTemp) {
		return (startFrame + curFrame);
	}
	else {
		return (tempFrame);
	}
}
