#include "SpriteManager.h"

SpriteManager::SpriteManager(char *dir) {
	/*
		Create a new SpriteManager based on the sequentially numbered
		contents of a directory.

		Files should be named [0..n].png
	*/
	char *filename = new char[200];
	bool bDone = false;
	int spr = 0;
	
	while(!bDone) {
		sprintf(filename, "%s/%i.png", dir, spr);
		if(addSprite(filename) < 0) {
			printf("Loaded %i sprites\n", spr);
			bDone = true;
		}
		spr++;
	}
	
	delete[] filename;
	
	screenWidth = 100000;
	screenHeight = 100000;
}

int SpriteManager::getScreenWidth() {
	return screenWidth;
}

int SpriteManager::getScreenHeight() {
	return screenHeight;
}

void SpriteManager::setScreenSize(int w, int h) {
	/*
		Set the width and height of the viewport
		to enable more intelligent culling by
		draw()
	*/
	screenWidth = w;
	screenHeight = h;
}

int SpriteManager::addSprite(char *filename) {
	/* Add a sprite from 'filename' and return the ID that should be used
	   to reference it in further use.

	   Returns -1 on the event of an error!
	*/
	SDL_Surface* sprite = IMG_Load(filename);
	if(!sprite) {
		fprintf(stderr, "Sorry, sprite %s is missing\n", filename);
		return -1;
	}
	
	GLuint* glTex = new GLuint;
	glGenTextures(1, glTex);
	SDL_SetAlpha(sprite, 0, 255);
	SDL_Surface* destImage = SDL_CreateRGBSurface(SDL_SWSURFACE,
			  sprite->w, sprite->h,
			  32, SDL_SwapLE32(0x000000FF),
			  SDL_SwapLE32(0x0000FF00),
			  SDL_SwapLE32(0x00FF0000),
			  SDL_SwapLE32(0xFF000000));
	
	if(!destImage) {
		fprintf(stderr, "FAIL %s, %i) New surface couldn't be created for %s. Likely cause: out of RAM\n", __FILE__, __LINE__, filename);
		exit(1);
	}
	
	SDL_BlitSurface(sprite, NULL, destImage, NULL);
	glBindTexture(GL_TEXTURE_2D, *glTex);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	gluBuild2DMipmaps(GL_TEXTURE_2D, 4, destImage->w, destImage->h, GL_RGBA,
					  GL_UNSIGNED_BYTE, destImage->pixels);
	SDL_FreeSurface(sprite);
	SDL_FreeSurface(destImage);
	
	sprites.push_back(glTex);
	return (int)(sprites.size() - 1);
}

GLuint* SpriteManager::getSprite(int num) {
	/*
		Return the intended sprite, or NULL in case of failure
	*/
	if(num < 0 || num >= (int)sprites.size()) {
		fprintf(stderr, "Requested sprite %i is out of bounds\n", num);
		return NULL;
	}
	return sprites[num];
}

void SpriteManager::freeSprites() {
	/*
		Empty the sprite buffer, and attempt to de-allocate
		memory. There is probably a routine that needs to
		be called from OpenGL to clear the actual vram, but
		we will be OK with this
	*/
	for(unsigned int i = 0; i < sprites.size(); i++) {
		delete sprites[i];
	}
	sprites.clear();
}

void SpriteManager::drawSprite(int spriteNum, int targetX, int targetY, int width, int height, float rotation) {
	
	if(targetX == -1337 && targetY == -1337) {
		// Centre it!
		targetX = screenWidth/2;
		targetY = screenHeight/2;
	}
	
	if(targetX < -width || targetX >= screenWidth || targetY < -height || targetY >= screenHeight) {
		// Can't even draw a piece of this.
		return;
	}
	
	glPushMatrix();
	// Could be faster with a bit-shift.
	GLfloat half_width = (GLfloat)(width/2);
	GLfloat half_height = (GLfloat)(height/2);
	// Translate before we rotate, to make up for the fuckery
	glTranslatef((float)(targetX + half_width),
			(float)(targetY + half_height), 0.0f);
	// Rotate the coordinate system on z
	glRotatef((float)(180.0f - rotation), 0, 0, 1);
	// Set up our alpha blending
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	// Put the texture into the quad
	GLuint* sprite = getSprite(spriteNum);
	if(!sprite) {
		fprintf(stderr, "FAIL) Requested dead sprite %i to be drawn at (%i,%i)\n", spriteNum, targetX, targetY);
		return;
	}
	glBindTexture(GL_TEXTURE_2D, *sprite);
	// Draw quad
	glBegin(GL_QUADS);
	glTexCoord2f(0.0f, 0.0f);
	glVertex2f(half_width, half_height);
	glTexCoord2f(1.0f, 0.0f);
	glVertex2f(-1*half_width, half_height);
	glTexCoord2f(1.0f, 1.0f);
	glVertex2f(-1*half_width, -1*half_height);
	glTexCoord2f(0.0f, 1.0f);
	glVertex2f(half_width, -1*half_height);
	glEnd();
	// Reset back to normal.
	glPopMatrix();
}

int SpriteManager::size() {
	return (int)sprites.size();
}

void SpriteManager::drawSprite(int spriteNum, int targetX, int targetY, int width, int height) {
	drawSprite(spriteNum, targetX, targetY, width, height, 0.0f);
}

SpriteManager::~SpriteManager() {
	freeSprites();
}
